@@ -10,8 +10,8 @@ this:
  >>> a = HandwrittenData(...)
  >>> feature_list = [features.StrokeCount(),
                     features.ConstantPointCoordinates(strokes=4,
-                                                        points_per_stroke=20,
-                                                        fill_empty_with=0)
+                                                      points_per_stroke=20,
+                                                      fill_empty_with=0)
                     ]
  >>> x = a.feature_extraction(feature_list)
 """
@@ -29,8 +29,8 @@ from shapely.geometry import LineString
 import itertools
 import numpy
 # mine
-import HandwrittenData
-import preprocessing
+import hwrt.HandwrittenData as HandwrittenData
+import hwrt.preprocessing as preprocessing
 
 
 def get_class(name):
@@ -92,11 +92,11 @@ class ConstantPointCoordinates(object):
 
     """Take the first `points_per_stroke=20` points coordinates of the first
        `strokes=4` strokes as features. This leads to
-       $2 \cdot \text{points\_per\_stroke} \cdot \text{strokes}$ features.
+       :math:`2 \cdot \text{points\_per\_stroke} \cdot \text{strokes}` features.
 
-       If `points` is set to $0$, the first `points\_per\_stroke` point
+       If `points` is set to 0, the first `points\_per\_stroke` point
        coordinates and the \verb+pen_down+ feature is used. This leads to
-       $3 \cdot \text{points_per_stroke}$ features."""
+       :math:`3 \cdot \text{points_per_stroke}` features."""
 
     normalize = False
 
@@ -153,7 +153,7 @@ class ConstantPointCoordinates(object):
                             x.append(self.fill_empty_with)
                             x.append(self.fill_empty_with)
                 else:
-                    for i in range(self.points_per_stroke):
+                    for _ in range(self.points_per_stroke):
                         x.append(self.fill_empty_with)
                         x.append(self.fill_empty_with)
         else:
@@ -179,7 +179,9 @@ class ConstantPointCoordinates(object):
 
 class FirstNPoints(object):
 
-    """First n points a 2*n dimensional feature."""
+    """Similar to the `ConstantPointCoordinates` feature, this feature takes the
+       first `n=81` point coordinates. It also has the `fill_empty_with=0` to
+       make sure that the dimension of this feature is always the same."""
 
     normalize = False
 
@@ -246,7 +248,7 @@ class StrokeCount(object):
 
 class Bitmap(object):
 
-    """n x n bitmap or the recording."""
+    """n × n grayscale bitmap of the recording."""
 
     normalize = True
 
@@ -355,8 +357,10 @@ class Width(object):
 
     """Width of a recording as a 1 dimensional feature.
 
-    Note that this is the current Width. so if you scaled the recording, this
-    will not be the original height.
+    .. note::
+
+        This is the current width. So if the recording was scaled, this will
+        not be the original width.
     """
 
     normalize = True
@@ -381,8 +385,10 @@ class Height(object):
 
     """Height of a recording as a a 1 dimensional feature.
 
-    Note that this is the current hight. so if you scaled the recording, this
-    will not be the original height.
+    .. note::
+
+        This is the current hight. So if the recording was scaled, this will not
+        be the original height.
     """
 
     normalize = True
@@ -405,7 +411,8 @@ class Height(object):
 
 class Time(object):
 
-    """Time it took to create the recording as a 1 dimensional feature."""
+    """The time in milliseconds it took to create the recording. This is a 1
+       dimensional feature."""
 
     normalize = True
 
@@ -416,6 +423,8 @@ class Time(object):
         return "Time"
 
     def get_dimension(self):
+        """Get the dimension of the returned feature. This equals the number
+           of elements in the returned list of numbers."""
         return 1
 
     def __call__(self, handwritten_data):
@@ -427,7 +436,7 @@ class Time(object):
 
 class CenterOfMass(object):
 
-    """Center of mass as a 2 dimensional feature for a recording."""
+    """Center of mass of a recording as a 2 dimensional feature."""
 
     normalize = True
 
@@ -492,7 +501,11 @@ class StrokeCenter(object):
 
 
 class StrokeIntersections(object):
-    """Count the number of intersections the symbol has.
+    """Count the number of intersections which strokes in the recording have
+       with each other in form of a symmetrical matrix for the first
+       `stroke=4` strokes. The feature dimension is
+       :math:`round \frac{\text{strokes}^2}{2} + \frac{\text{strokes}}{2}`,
+       because the symmetrical part is discarded.
 
     =======   ======= ======= ======= ===
       -       stroke1 stroke2 stroke3
@@ -587,12 +600,12 @@ class StrokeIntersections(object):
                                                      pointlist[j]))
         # print("%s - %i" % (handwritten_data.formula_in_latex,
         #                    handwritten_data.raw_data_id))
-        curr = 0
-        for i in range(self.strokes, 0, -1):
-            line = "\t"*(self.strokes - i)
-            for j in range(i):
-                line += str(x[curr]) + "\t"
-                curr += 1
+        # curr = 0
+        # for i in range(self.strokes, 0, -1):
+        #     line = "\t"*(self.strokes - i)
+        #     for j in range(i):
+        #         line += str(x[curr]) + "\t"
+        #         curr += 1
 
         assert self.get_dimension() == len(x), \
             "Dimension of %s should be %i, but was %i" % \
@@ -602,7 +615,10 @@ class StrokeIntersections(object):
 
 class ReCurvature(object):
 
-    """Re-curvature is a 1 dimensional, global feature for a recording."""
+    """Re-curvature is a 1 dimensional, stroke-global feature for a recording.
+       It is the ratio
+       :math:`\frac{\text{height}(s)}{\text{distance}(s[0], s[-1])}`.
+    """
 
     normalize = True
 
